% Example of Point Source Recovery, 2D FBP and MLEM Algorithms
% Original: UTU Signal and Image Processing Course, 12.5.2022
% 
% Modified for Octave from MATLAB on 9.5.2018
% Jarmo Teuho 2022
close all;clear all

%% Noisless vs noisy case

% Introduce noise? (1 = yes, 0 = no)
noise=0;

% Create phantom
IM_ref = phantom(128); 

phi = 0:(180/128):(180-1/(128+1)); % all angles

% Forward project to create simulated, measured projection data
[p_measured,xp] = radon(IM_ref,phi); 

% Add noise to the projection data
randp('state',1)
p_measured_noisy = poissrnd(p_measured);

%% Visualize data data with FBP and MLEM

% Do backprojection and apply Ramp filter
if noise

  f_FBP_ramp=iradon(p_measured_noisy,phi,'Linear','Hann',1,128); % noisy case

else

  f_FBP_ramp=iradon(p_measured,phi,'Linear','Ram-Lak',1,128); % noisless case

end

% Change from 2 to 8 (or large number)
ITER=6;

% Do MLEM reconstruction 
% Create system model H and initial image f
s = iradon(ones(128,128),phi,'Linear','None',1,128); % system model
f = ones(128,128); % initial image

% Run MLEM
if noise
  
  f_MLEM=mlem(ITER,p_measured_noisy,f,s,phi); % noisy case

 else
 
  f_MLEM=mlem(ITER,p_measured,f,s,phi);  % noisless case

 end
  
% Visualize result
figure;
subplot(2,2,1),imshow(IM_ref,[]),title('Reference object'),colorbar
subplot(2,2,2),imshow(p_measured',[]),title('Sinogram'),colorbar
subplot(2,2,4),imshow(f_MLEM,[]),title('MLEM '),colorbar
subplot(2,2,3),imshow(f_FBP_ramp,[]),title('FBP'),colorbar
set(gcf,'Position',[67 70 916 722])

%% Measure convergence from reconstructed image via ROI analysis and compare

% Select where to draw the target ROI and it's size
x=66
y=44
r=3

% Create ROI
mask_target = make_circular_VOI(x,y,r,128,128);

% Select where to draw the background ROI and it's size
x=66
y=18
r=3

% Create ROI
mask_bg = make_circular_VOI(x,y,r,128,128);

% Visualize ROI
figure;
subplot(1,3,1),imshow(IM_ref), title('Phantom')
subplot(1,3,2),imshow(mask_target), title('Target ROI')
subplot(1,3,3),imshow(mask_bg), title('Bakcground ROI')

% Convergence evalaution
for ITER=1:25
    
    % Run MLEM
    if noise
  
      f_MLEM=mlem(ITER,p_measured_noisy,f,s,phi); % noisy case
    
    else
    
      f_MLEM=mlem(ITER,p_measured,f,s,phi); % noiseless case
    
    end
    
    % Apply Gaussian filter
    % f_MLEM=imsmooth(f_MLEM,'Gaussian',1.25); 
   
    % Save VOI average for target and background VOI
    VOI_results_target_mean(:,ITER)=mean(f_MLEM(mask_target));
    VOI_results_background_mean(:,ITER)=mean(f_MLEM(mask_bg));
  
     % Save VOI average for target and background VOI
    VOI_results_target_sd(:,ITER)=std(f_MLEM(mask_target));
    VOI_results_background_sd(:,ITER)=std(f_MLEM(mask_bg));
    
end

% Check last iteration image
%figure;
%imshow(f_MLEM), title(strcat('Iter: ',num2str(ITER)))

% Get FBP resuls
VOI_FBP_target_mean=mean(f_FBP_ramp(mask_target));
VOI_FBP_background_mean=mean(f_FBP_ramp(mask_bg));

% Save VOI average for target and background VOI
VOI_FBP_target_sd=std(f_FBP_ramp(mask_target));
VOI_FBP_background_sd=std(f_FBP_ramp(mask_bg));

% Plot results
figure;plot(VOI_results_target_mean,'.-')
hold on
plot(VOI_FBP_target_mean*ones(1,max(ITER)),'--')
hold on
plot(VOI_results_background_mean,'o-')
plot(VOI_FBP_background_mean*ones(1,max(ITER)),'.x')
legend('MLEM Target','FBP Target','MLEM Background','FBP Background')
title('VOI convergence')

%% Calculate TBR and SNR 
TBR_MLEM=VOI_results_target_mean./VOI_results_background_mean
TBR_FBP=VOI_FBP_target_mean./VOI_FBP_background_mean

figure;plot(TBR_MLEM,'.x')
hold on
plot(TBR_FBP*ones(1,max(ITER)),'--')
legend('MLEM','FBP')
title('Target to background ratio (TBR)')

SNR_MLEM=VOI_results_target_mean./VOI_results_background_sd
SNR_FBP=VOI_FBP_target_mean./VOI_FBP_background_sd

figure;plot(SNR_MLEM,'.x')
hold on
plot(SNR_FBP*ones(1,max(ITER)),'--')
legend('MLEM','FBP')
title('Signal to noise ratio (SNR)')

%% Calculate recovery coefficient (RC)

VOI_ref=mean(IM_ref(mask_target));

RC_MLEM=VOI_results_target_mean./VOI_ref
RC_FBP=VOI_FBP_target_mean./VOI_ref

figure;plot(RC_MLEM)
hold on
plot(RC_FBP*ones(1,max(ITER)))
legend('MLEM','FBP')
title('VOI recovery coefficient')